"""
Class handling requests to remote resources.
"""
import contextlib
import urllib.request
from http import HTTPStatus

from podmena.utils import _info


class SimpleFetcher(object):
    def __init__(self, url):
        self.url = url

    def request(self, url=None):
        if url is None:
            url = self.url
        _info(f"Requesting {url}")

        req = urllib.request.Request(url)
        with contextlib.closing(urllib.request.urlopen(req)) as resp:
            if resp.status != HTTPStatus.OK:
                raise RuntimeError("Got wrong status %s" % resp.status)
            return resp.read().decode("utf-8")



import os
import sys
import shutil

import click

from podmena.fetcher import SimpleFetcher
from podmena.parser import RegexParser
from podmena.group import AliasedGroup
from podmena.utils import (
    _warn,
    _note,
    _info,
    force_symlink,
    get_git_root_dir,
    set_git_global_hooks_path,
    get_git_config_hooks_value,
    unset_git_global_hooks_path,
)


CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))
RESOURCES_DIR = os.path.join(CURRENT_DIR, "resources")
HOOK_FILENAME = "commit-msg"
DATABASE_FILE = "emoji-db"


@click.group(cls=AliasedGroup)
@click.version_option()
def cli():
    pass


@cli.command(
    name="status",
    help="Shows whether podmena is installed for current repository or globally",
)
def status():
    active = False
    git_root_dir = get_git_root_dir()
    if git_root_dir is not None:
        local_hooks_path = os.path.join(git_root_dir, ".git", "hooks")
        database_path = os.path.join(local_hooks_path, DATABASE_FILE)
        hook_path = os.path.join(local_hooks_path, HOOK_FILENAME)
        if os.path.exists(database_path) and os.path.exists(hook_path):
            _note("podmena is activated for current repository!")
            active = True

    global_hooks_path = os.path.expanduser("~/.podmena/hooks")
    global_database_path = os.path.join(global_hooks_path, DATABASE_FILE)
    global_hook_path = os.path.join(global_hooks_path, HOOK_FILENAME)
    git_global_hooks_config = get_git_config_hooks_value()

    if (
        os.path.exists(global_database_path)
        and os.path.exists(global_hook_path)
        and git_global_hooks_config == global_hooks_path
    ):
        _note("podmena is activated globally!")
        active = True

    if not active:
        _warn("podmena is not activated neither for current repository nor globally!")


@cli.command(
    name="update", help="Update database of emoji in case of any changes from remote"
)
def grab():
    url = "https://www.webpagefx.com/tools/emoji-cheat-sheet/"
    fetcher = SimpleFetcher(url=url)
    parser = RegexParser()
    html = fetcher.request()
    emoji = parser.parse(html)
    database_path = os.path.join(RESOURCES_DIR, DATABASE_FILE)
    with open(database_path, "w") as f:
        f.write("\n".join(emoji))
    _note("Downloaded {} emoji to database".format(len(emoji)))


@cli.group(
    name=("add", "activate", "enable", "install", "on"),
    help="Activate podmena",
)
def install():
    pass


@install.command(
    name="local",
    help="Install podmena for current git repository",
)
def local_install():
    git_local_root = os.path.join(os.getcwd(), ".git")
    local_hooks_path = os.path.join(git_local_root, "hooks")
    if os.path.exists(git_local_root) and os.path.isdir(git_local_root):
        src_file = os.path.join(RESOURCES_DIR, HOOK_FILENAME)
        dst_file = os.path.join(local_hooks_path, HOOK_FILENAME)
        shutil.copyfile(src_file, dst_file)
        os.chmod(dst_file, 0o0775)
        db_file = os.path.join(RESOURCES_DIR, DATABASE_FILE)
        db_link = os.path.join(local_hooks_path, DATABASE_FILE)
        force_symlink(db_file, db_link)
        _note("Successfully installed for current repository!")
    else:
        _warn("Not a git repository")
        sys.exit(1)


@install.command(
    name="global",
    help="Install podmena globally for every git repository",
)
def global_install():
    global_hooks_path = os.path.expanduser("~/.podmena/hooks")
    confirm_info = (
        "This will set one global hooks directory for all you repositories.\n"
        "This action may deactivate your previous hooks installed per "
        "repository.\nFor more info see "
        "https://git-scm.com/docs/git-config#git-config-corehooksPath\n"
    )
    _info(confirm_info)
    if click.confirm("Do you want to continue?", abort=True):
        if not os.path.exists(global_hooks_path):
            os.makedirs(global_hooks_path)

        src_file = os.path.join(RESOURCES_DIR, HOOK_FILENAME)
        dst_file = os.path.join(global_hooks_path, HOOK_FILENAME)
        shutil.copyfile(src_file, dst_file)
        os.chmod(dst_file, 0o0775)
        db_file = os.path.join(RESOURCES_DIR, DATABASE_FILE)
        db_link = os.path.join(global_hooks_path, DATABASE_FILE)
        force_symlink(db_file, db_link)
        set_git_global_hooks_path(global_hooks_path)
        _note("Installed globally for all repos")


@cli.group(
    name=("rm", "remove", "delete", "deactivate", "disable", "off", "uninstall"),
    help="Deactivate podmena",
)
def remove():
    pass


@remove.command(
    name="local",
    help="Uninstall podmena for current git repository",
)
def local_uninstall():
    git_local_root = os.path.join(os.getcwd(), ".git")
    hook_filepath = os.path.join(git_local_root, "hooks", HOOK_FILENAME)
    db_link = os.path.join(git_local_root, "hooks", DATABASE_FILE)
    if os.path.exists(hook_filepath):
        os.remove(hook_filepath)
        os.remove(db_link)
        _note("Uninstalled for current repository")
    else:
        _warn("podmena is not installed for current repository!")
        sys.exit(1)


@remove.command(
    name="global",
    help="Uninstall podmena globally",
)
def global_uninstall():
    rc = unset_git_global_hooks_path()
    if rc == 0:
        _info("Deactivated podmena globally")
    elif rc == 5:
        _warn("podmena is not installed globally!")
        sys.exit(1)
    else:
        _warn("Failed to deactivate")
        sys.exit(rc)


if __name__ == "__main__":
    cli()



import re


class RegexParser(object):
    def __init__(self):
        self.pattern = re.compile(r"<span .+></span>:<span .+>([\w_]+)</span>:</div>")

    def parse(self, text):
        return self.pattern.findall(text)



import os
import subprocess

import click


def _warn(message):
    click.secho(message, fg="red")


def _note(message):
    click.secho(message, fg="green")


def _info(message):
    click.secho(message, fg="yellow")


def set_git_global_hooks_path(hooks_path):
    subprocess.call(
        [
            "git",
            "config",
            "--global",
            "core.hooksPath",
            hooks_path,
        ]
    )


def get_git_config_hooks_value():
    try:
        return (
            subprocess.check_output(
                [
                    "git",
                    "config",
                    "--get",
                    "core.hooksPath",
                ]
            )
            .decode()
            .strip()
        )
    except subprocess.CalledProcessError:
        pass


def unset_git_global_hooks_path():
    return subprocess.call(
        [
            "git",
            "config",
            "--global",
            "--unset",
            "core.hooksPath",
        ]
    )


def get_git_root_dir():
    try:
        return (
            subprocess.check_output(
                [
                    "git",
                    "rev-parse",
                    "--show-toplevel",
                ],
                stderr=subprocess.STDOUT,
            )
            .decode()
            .strip()
        )
    except subprocess.CalledProcessError:
        pass


def force_symlink(src, dst):
    if os.path.exists(dst) and os.path.islink(dst):
        os.remove(dst)

    os.symlink(src, dst)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['podmena']

package_data = \
{'': ['*'], 'podmena': ['resources/*']}

install_requires = \
['click>=8.0,<9.0']

entry_points = \
{'console_scripts': ['podmena = podmena.cli:cli']}

setup_kwargs = {
    'name': 'podmena',
    'version': '0.5.1',
    'description': 'Enhance your commit messages with emoji',
    'long_description': "## podmena\n\n![Checks](https://github.com/bmwant/podmena/actions/workflows/tests.yml/badge.svg)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nEnhance your commit messages with emoji 🍒\n\npodmena will automatically add random emoji to every commit message for any\ngit repository installed.\n\n![emoji](podmena.png)\n\n873 items in database so far!\n\nCredits go to [WebpageFX](https://www.webpagefx.com/tools/emoji-cheat-sheet/)\nfor list of emoji!\n\n### Installation\n\n```bash\n$ pip install podmena\n```\n\n* Activate for current git repository\n\n```bash\n$ podmena add local\n```\n\nYou can also replace `add` with a different alias\n\n`activate` / `enable` / `install` / `on`\n\ne.g. `podmena enable local`\n\n* Activate globally for all repositories (works with git `2.9.1` and above)\n\n```bash\n$ git --version\n$ podmena add global  # Aliases work here as well\n```\n\n* Deactivate it\n```bash\n$ podmena rm local\n$ podmena rm global\n```\n\nYou can replace `rm` with any of these available aliases\n\n`remove` / `delete` / `deactivate` / `disable` / `off` / `uninstall`\n\ne.g. `podmena deactivate local`\n\n* Check current status if you not sure\n\n```bash\n$ podmena status\n```\n\n* And finally `podmena --version` and `podmena --help` in case you need more\ndetails.\n\n> **NOTE:** uninstalling globally will not remove hooks from repositories where\nit was installed locally. You need to switch to that directory manually and uninstall it locally as well.\n\n### Contribute\n\nSee [DEVELOP.md](DEVELOP.md) to setup your local development environment and feel free to create a pull request with a new feature.\n\n### Releases\n\nSee [CHANGELOG.md](CHANGELOG.md) for the new features included within each release.\n\n### See also\n\n* [GitHooks](https://githooks.com/)\n* [Atlassian tutorial for git hooks](https://www.atlassian.com/git/tutorials/git-hooks)\nThanks [@kakovskyi](https://github.com/kakovskyi) working for Atlassian!\n* It's a wrong place to search if you are looking for 🍋 lemonparty.fun 🍋 club\n\n### Say thanks!\n\n🐶 `D7DA74qzZUyh9cctCxWovPTEovUSjGzL2S` this is [Dogecoin](https://dogecoin.com/) wallet to support the project.\n",
    'author': 'Misha Behersky',
    'author_email': 'bmwant@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bmwant/podmena',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



import typing as t

import click


class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv

        commands = self.list_commands(ctx)
        for c in commands:
            if isinstance(c, tuple) and cmd_name in c:
                return click.Group.get_command(self, ctx, c)
        ctx.fail("No such command '{}'".format(cmd_name))

    def list_commands(self, ctx: click.Context) -> t.List[str]:
        return self.commands
